package net.wanji.opt.service.impl;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDataHistMapper;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.opt.bo.ProblemSchemeBO;
import net.wanji.opt.service.SchemeEvaluateService;
import net.wanji.opt.vo.SchemeEvaluateProblemSchemeVO;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/8/25 16:09
 */
@Slf4j
@Service
public class SchemeEvaluateServiceImpl implements SchemeEvaluateService {
    private final CrossDataHistMapper crossDataHistMapper;

    public SchemeEvaluateServiceImpl(CrossDataHistMapper crossDataHistMapper) {
        this.crossDataHistMapper = crossDataHistMapper;
    }

    @Override
    public List<SchemeEvaluateProblemSchemeVO> problemSchemeList(ProblemSchemeBO problemSchemeBO) {
        List<SchemeEvaluateProblemSchemeVO> res = new ArrayList<>();
        String crossId = problemSchemeBO.getCrossId();
        Date startDate = problemSchemeBO.getStartDate();
        Date endDate = problemSchemeBO.getEndDate();
        endDate = DateUtil.offsetDay(endDate, 1); // 包含最后一天
        int startDateStamp = (int) (startDate.getTime() / 1000); // 10位时间戳
        int endDateStamp = (int) (endDate.getTime() / 1000);
        String startTime = problemSchemeBO.getStartTime();
        String endTime = problemSchemeBO.getEndTime();
        List<CrossDataHistPO> crossDataHistPOList =
                crossDataHistMapper.selectByCrossIdAndStartEnd(crossId, startDateStamp, endDateStamp);
        // 根据入参筛选结果
        Map<String, List<CrossDataHistPO>> stringListMap = getStringListMap(
                crossDataHistPOList, endTime, startTime);




        return res;
    }

    @NotNull
    private static Map<String, List<CrossDataHistPO>> getStringListMap(List<CrossDataHistPO> crossDataHistPOList, String endTime, String startTime) {
        // 过滤有问题的记录
        List<CrossDataHistPO> filteredList = crossDataHistPOList.stream()
                .filter(po -> po.getStatus() != null && po.getStatus() != 0)
                .collect(Collectors.toList());
        // 保留 duration 最大的一条
        List<CrossDataHistPO> result = filteredList.stream()
                .collect(Collectors.groupingBy(CrossDataHistPO::getStartTime))
                .values().stream()
                .map(list -> list.stream().max(Comparator.comparingInt(CrossDataHistPO::getDuration)).orElse(null))
                .collect(Collectors.toList());
        // 按照 startTime 和 endTime 的 "HH:mm" 的时间区间分组
        Map<String, List<CrossDataHistPO>> groupedByTimeInterval = result.stream()
                .collect(Collectors.groupingBy(po -> {
                    Date startTime1 = po.getStartTime();
                    Date endTime1 = new Date(startTime1.getTime() + po.getDuration() * 60 * 1000);
                    SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
                    return sdf.format(startTime1) + " - " + sdf.format(endTime1);
                }));
        // 过滤 groupedByTimeInterval，保留与 startTime 和 endTime 有交叉的条目
        Map<String, List<CrossDataHistPO>> filteredGroupedByTimeInterval = groupedByTimeInterval.entrySet().stream()
                .filter(entry -> {
                    String[] times = entry.getKey().split(" - ");
                    String groupStartTime = times[0];
                    String groupEndTime = times[1];
                    return (groupStartTime.compareTo(endTime) < 0 || groupStartTime.equals(endTime))
                            && (groupEndTime.compareTo(startTime) > 0 || groupEndTime.equals(startTime));
                })
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        return filteredGroupedByTimeInterval;
    }
}
